<?php

namespace Slovocast\Infrastructure\Session;

use Odan\Session\SessionInterface;

class FlashMessages implements FlashMessagesInterface
{
    const LEVEL_ERROR = "error";
    const LEVEL_NOTICE = "notice";
    const LEVEL_SUCCESS = "success";

    private string $key = "_flash";

    public function __construct(
        protected SessionInterface $session
    ) {
        if (!$this->session->has($this->key)) {
            $this->session->set($this->key, []);
        }
    }

    /**
     * @return array<string, array<string>>
     */
    protected function getMessages(): array
    {
        return $this->session->get($this->key);
    }

    /**
     * @param array<string, array<string>> $messages
     * @return void
     */
    protected function saveMessages(array $messages): void
    {
        $this->session->set($this->key, $messages);
    }
    
    public function add(string $level, string $message): void
    {
        $messages = $this->getMessages();
        $messages[$level][] = $message;
        $this->saveMessages($messages);
    }

    public function get(string $level): array
    {
        $messages = $this->getMessages();

        if (!array_key_exists($level, $messages)) {
            return [];
        }

        $buffer = $messages[$level];
        unset($messages[$level]);
        $this->saveMessages($messages);
        return $buffer;
    }

    public function getAll(): array
    {
        $messages = $this->getMessages();
        $this->clear();
        return $messages;
    }

    public function clear(?string $level = null): void
    {
        if (!$level) {
            $this->session->set($this->key, []);
        } else {
            $messages = $this->getMessages();

            if (array_key_exists($level)) {
                unset($messages[$level]);
            }

            $this->saveMessages($messages);
        }
    }
}
